#!/usr/local/bin/perl 
package PNCE::FakeCAS;
#
#Module for a fake CAS server which allows you to be anyone
#For testing of apps using CAS authentication
#
#Pure OO interface
#    Written by Tom Payerle, <payerle@physics.umd.edu>, 2006
#    Copyright 2006 by University of Maryland
#    This library is free software; you can redistribute it and/or modify
#    it under the same terms as Perl itself.
                                                                                

use strict;

use CGI;
my $cgi = new CGI;
my $cookie_name = 'FakeCASTGT';


sub new($)
{	my $class = shift;
	my $obj = {};
	bless $obj, $class;
	return $obj;
}

sub cgi($)
{	my $self =shift;
	return $cgi;
}

sub tgc_name($)
{	my $self = shift;
	return 'FAKECASTGC';
}

sub fatal_error($$;$)
#Make fatal error, logging main reason to stderr, giving a stripped down
#version to user
{	my $self = shift;
	my $logmsg = shift;
	my $usermsg = shift || 'A fatal error occured in this application.';

	my $now = localtime;
	my $ip = $ENV{REMOTE_ADDR};
	my $header = "$now FakeCAS [$ip]:";
	warn("$header $logmsg");

	print $cgi->header(), $cgi->start_html('Fatal Error');
	print <<EOF;
<h1>Fatal Error</h1>
<p>We are unable to process your request due to a fatal error.
<ul>
<li>$usermsg
</ul>
EOF
	print $cgi->end_html;
	exit 0;
}

sub make_string_from_flags($@)
#Given a list of flags (in key=>value format), returns a string representing
#set flags.  Used by create_ticket routines
{	my $self = shift;
	my @flags = @_;

	my $flagstr ='';
	my %temp = @flags;
	if ( $temp{warn} )
	{	$flagstr .= 'w';
	}
	delete $temp{warn};
	if ( $temp{debug} )
	{	$flagstr .= 'd';
	}
	delete $temp{debug};
	if ( $temp{forced} )
	{	$flagstr .= 'f';
	}
	delete $temp{forced};
	$self->fatal_error("Illegal flags given: (" . (join ", ", keys %temp) 
		.  ")" ) if scalar(%temp);

	return $flagstr;
}

sub make_flags_from_string($$)
#Given a string representing a set of flags (e.g. from make_string_from_flags),
#returns a ref to hash representing the flags.  Key is flag name, value is
#flag value
{	my $self = shift;
	my $flagstr = shift;
	
	my $flags = {	warn=>0,
			debug=>0,
		};
	$flags->{warn} = 1 if ( $flagstr=~/w/ );
	$flags->{debug} = 1 if ( $flagstr=~/d/ );
	$flags->{forced} = 1 if ( $flagstr=~/f/ );
	return $flags;
}

sub valid_username($$)
#Validates username
#Returns true on success, false otherwise
{	my $self = shift;
	my $user = shift;
	return $user=~/^[\w\-]+$/;
}

sub parse_tgt($$)
#Given a ticket granting ticket, parses it.  Returns ( $username, \%flags)
#Returns (undef, undef ) if not valid
{	my $self =shift;
	my $ticket = shift;

	my ($junk, $user, $flagstr) = split /:/, $ticket;
	return (undef, undef) unless $junk eq 'FakeCAS';
	return (undef, undef) unless $self->valid_username($user);
	my $flags = $self->make_flags_from_string($flagstr);

	return ($user, $flags);
}


sub create_tgt_for_user($$@)
#Creates  Ticket-granting ticket for user user with flags @flags
#Returns string with ticket (which then needs to be put in cookie)
{	my $self = shift;
	my $user = shift;
	my @flags = @_;

	my $flagstr = $self->make_string_from_flags(@flags);
	$self->fatal_error("Illegal username given: $user") 
		unless $self->valid_username($user);

	my $ticket="FakeCAS:$user:$flagstr";
	return $ticket;
}

sub parse_st($$)
#Given a service ticket, parses it.  Returns ( $username, \%flags)
#Returns undef, undef if not valid
{	my $self =shift;
	my $ticket = shift;

	my ($junk, $flagstr, $user) = split /-/, $ticket, 3;
	return (undef, undef) unless $junk eq 'ST';
	return (undef, undef) unless $self->valid_username($user);

	my $flags = $self->make_flags_from_string($flagstr);
	return ($user, $flags);
}

sub create_st_for_user($$@)
#Creates  Ticket-granting ticket for user user with flags @flags
#Returns string with ticket (which then needs to be put in cookie)
{	my $self = shift;
	my $user = shift;
	my @flags = @_;

	my $flagstr = $self->make_string_from_flags(@flags);
	$self->fatal_error("Illegal username given: $user") 
		unless $self->valid_username($user);

	my $ticket="ST-$flagstr-$user";
	return $ticket;
}

sub url_with_args($$@)
#Append args to $url
#
#BUG: Does not do URL encoding of args
{	my $self = shift;
	my $newurl = shift;
	my @args = @_;

	my ($key, $val);
	my $connector = '?';
	while ($key = shift @args)
	{	$val = shift @args;
		$newurl .= "$connector$key=$val";
		$connector='&';
	}
	return $newurl;
}

sub bake_cookie($$)
#Given a tgt value, returns a new cookie
#Special case, if tgt eq '*DELETE*', creates empty cookie
{	my $self = shift;
	my $tgt = shift;
	$tgt = '' if $tgt eq '*DELETE*';

	my $path = $cgi->url(-absolute=>1);
	$path=~s/FakeCAS.*$/FakeCAS/;
	my $cookie = $cgi->cookie(
		-name => $cookie_name,
		-value => $tgt,
		-path => $path,
		);
	return $cookie;
}

sub get_cookie($)
{	my $self = shift;
	my $cookie = $cgi->cookie($cookie_name);
	return $cookie;
}
sub do_redirect($$;$$)
#Takes service URL, tgt, and st, and redirects to $url, setting TGT cookie
#if tgt given and appending ticket=$st to url if st given
#Special case: tgt='*DELETE*' means to empty the cookie
{	my $self = shift;
	my $url = shift;
	my $tgt = shift;
	my $st = shift;

	my @redirect_args = ();
	if ( $tgt )
	{	my $cookie = $self->bake_cookie($tgt);
		push @redirect_args, -cookie=>$cookie;
	}
	my @url_args = ();
	if ( $st )
	{	push @url_args, ticket=>$st;
	}

	my $newurl = $self->url_with_args($url,@url_args);
	print $cgi->redirect($newurl,@redirect_args);
	exit 0;
}
	

